// *****************************************************************************************
// SequencePathExample.as
// 
// Copyright (c) 2007 Ryan Taylor | http://www.boostworthy.com
// 
// Permission is hereby granted, free of charge, to any person
// obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without
// restriction, including without limitation the rights to use,
// copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following
// conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.
// *****************************************************************************************
// 
// +          +          +          +          +          +          +          +          +
// 
// *****************************************************************************************

// PACKAGE /////////////////////////////////////////////////////////////////////////////////

package
{
	// IMPORTS /////////////////////////////////////////////////////////////////////////////
	
	import flash.display.Sprite;
	import flash.display.StageAlign;
	import flash.display.StageScaleMode;
	import flash.events.Event;
	import flash.events.MouseEvent;
	
	import com.boostworthy.animation.easing.Transitions;
	import com.boostworthy.animation.rendering.RenderMethod;
	import com.boostworthy.animation.sequence.Timeline;
	import com.boostworthy.animation.sequence.tweens.PathTween;
	import com.boostworthy.core.Global;
	import com.boostworthy.geom.Path;
	
	// SWF /////////////////////////////////////////////////////////////////////////////////
	
	// Note: If using the Flash IDE to compile, these settings will be ignored.
	[SWF(backgroundColor="#111111", frameRate="31", width="550", height="400")]
	
	// CLASS ///////////////////////////////////////////////////////////////////////////////
	
	/**
	 * This example file provides you with an example of using the timeline object to
	 * animate a display object along a path. The 'Path' object presents an intuitive API
	 * for creating a path and optionally drawing it to a graphics container as well. 
	 * Simply launch the corresponding SWF file and click the mouse down to watch the 
	 * animation play forwards, then release the mouse button to watch it play in reverse.
	 * <p>
	 * Instructions for use:
	 * <p>
	 * Flash IDE users - this class is already set as the document class in the 
	 * corresponding FLA file for you. Simply open the FLA file and make any 
	 * changes to this file to experiment.
	 * <p>
	 * Non-Flash IDE users - Treat this class as your 'Main' class when you compile.
	 * 
	 * @see	com.boostworthy.animation.sequence.Timeline
	 * @see	com.boostworthy.geom.Path
	 */
	public class SequencePathExample extends Sprite
	{
		// *********************************************************************************
		// CLASS DECLERATIONS
		// *********************************************************************************
		
		// CONSTANTS ///////////////////////////////////////////////////////////////////////
		
		/**
		 * Configures the color of the box sprite.
		 */
		protected const BOX_COLOR:uint    = 0x006666;
		
		/**
		 * Configures the width of the box sprite.
		 */
		protected const BOX_WIDTH:Number  = 60;
		
		/**
		 * Configures the height of the box sprite.
		 */
		protected const BOX_HEIGHT:Number = 60;
		
		// MEMBERS /////////////////////////////////////////////////////////////////////////
		
		/**
		 * Holds an instance of the timeline object.
		 */
		protected var m_objTimeline:Timeline;
		
		/**
		 * Reference to the box sprite which will be created for the purpose
		 * of having something to animate.
		 */
		protected var m_spBox:Sprite;
		
		/**
		 * A path object to animate the box sprite along.
		 */
		protected var m_objPath:Path;
		
		// *********************************************************************************
		// EVENT HANDLERS
		// *********************************************************************************
		
		// CONSTRUCTOR /////////////////////////////////////////////////////////////////////
		
		/**
		 * Constructor.
		 */
		public function SequencePathExample()
		{
			// Initialize this object.
			init();
		}
		
		// MOUSE EVENTS ////////////////////////////////////////////////////////////////////
		
		/**
		 * Called when the mouse is clicked down.
		 * 
		 * @param	objEvent	An object containing information about the mouse event.
		 */
		protected function onMouseDown(objEvent:MouseEvent):void
		{
			// Play the timeline forwards.
			m_objTimeline.play();
		}
		
		/**
		 * Called when the mouse is clicked up.
		 * 
		 * @param	objEvent	An object containing information about the mouse event.
		 */
		protected function onMouseUp(objEvent:MouseEvent):void
		{
			// Play the timeline backwards.
			m_objTimeline.playReverse();
		}
		
		// *********************************************************************************
		// INTERNAL
		// *********************************************************************************
		
		// INIT ////////////////////////////////////////////////////////////////////////////
		
		/**
		 * Initializes this object.
		 */
		protected function init():void
		{
			// Set all default property values.
			setDefaultValues();
			
			// Store a global reference to the stage.
			// This is needed for objects to have reference to the enter
			// frame event.
			Global.stage = stage;
			
			// Create a new timeline object.
			m_objTimeline = new Timeline(RenderMethod.TIMER, 60);
			
			// Create a motion path for the animation.
			createMotionPath();
			
			// Create a box sprite to animate.
			createBox();
			
			// Create an animation for the box sprite.
			createAnimation();
			
			// Add event listeners for the 'mouse down' and 'mouse up' events.
			stage.addEventListener(MouseEvent.MOUSE_DOWN, onMouseDown);
			stage.addEventListener(MouseEvent.MOUSE_UP, onMouseUp);
		}
		
		/**
		 * Applies all default values to the corresponding properties.
		 */
		protected function setDefaultValues():void
		{
			// Apply the default values.
			stage.scaleMode = StageScaleMode.NO_SCALE;
			stage.align     = StageAlign.TOP_LEFT;
		}
		
		// GRAPHICS ////////////////////////////////////////////////////////////////////////
		
		/**
		 * Creates a box sprite to animate.
		 */
		protected function createBox():void
		{
			// Create a new sprite and draw a box using the configured box settings.
			m_spBox      = new Sprite();
			m_spBox.name = "m_spBox";
			m_spBox.graphics.beginFill(BOX_COLOR);
			m_spBox.graphics.drawRect(-BOX_WIDTH / 2, -BOX_HEIGHT / 2, BOX_WIDTH, BOX_HEIGHT);
			
			// Add the box to the display container.
			addChild(m_spBox);
			
			// Start the box off in the center of the screen.
			m_spBox.x = 0;
			m_spBox.y = Math.floor(stage.stageHeight / 2);
		}
		
		// ANIMATION ///////////////////////////////////////////////////////////////////////
		
		/**
		 * Creates a motion path for the animation.
		 */
		protected function createMotionPath():void
		{
			// Stage points to minimize calculations.
			var nX1:Number = Math.floor(stage.stageWidth / 2);
			var nX2:Number = Math.floor(stage.stageWidth / 4);
			var nY1:Number = Math.floor(stage.stageHeight / 2);
			
			// Create a new path and use the API to add curves.
			m_objPath = new Path();
			m_objPath.moveTo(0, nY1);
			m_objPath.curveTo(nX1 - nX2, stage.stageHeight, nX1, nY1);
			m_objPath.curveTo(nX1 + nX2, 0, stage.stageWidth, nY1);
			
			// OPTIONAL
			// 
			// This step isn't required for animation, however I chose to draw
			// the path as well as a visual reference for this example.
			var objGraphics:Sprite = new Sprite();
			objGraphics.graphics.lineStyle(1, 0x555555);
			m_objPath.draw(objGraphics.graphics);
			addChild(objGraphics);
		}
		
		/**
		 * Creates a sequence of animations for the box sprite using the timeline.
		 */
		protected function createAnimation():void
		{
			// Create the path tween and add it to the timeline.
			m_objTimeline.addTween(new PathTween(m_spBox, m_objPath, true, 10, 120, Transitions.SINE_IN_AND_OUT));
		}
	}
}