// *****************************************************************************************
// SequenceValueMapExample.as
// 
// Copyright (c) 2007 Ryan Taylor | http://www.boostworthy.com
// 
// Permission is hereby granted, free of charge, to any person
// obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without
// restriction, including without limitation the rights to use,
// copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following
// conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.
// *****************************************************************************************
// 
// +          +          +          +          +          +          +          +          +
// 
// *****************************************************************************************

// PACKAGE /////////////////////////////////////////////////////////////////////////////////

package
{
	// IMPORTS /////////////////////////////////////////////////////////////////////////////
	
	import flash.display.Sprite;
	import flash.display.StageAlign;
	import flash.display.StageScaleMode;
	import flash.events.Event;
	import flash.events.MouseEvent;
	
	import com.boostworthy.animation.easing.Transitions;
	import com.boostworthy.animation.rendering.RenderMethod;
	import com.boostworthy.animation.sequence.Timeline;
	import com.boostworthy.animation.sequence.tweens.AdvancedTween;
	import com.boostworthy.animation.sequence.tweens.PathTween;
	import com.boostworthy.core.Global;
	import com.boostworthy.geom.Path;
	
	// SWF /////////////////////////////////////////////////////////////////////////////////
	
	// Note: If using the Flash IDE to compile, these settings will be ignored.
	[SWF(backgroundColor="#111111", frameRate="31", width="550", height="400")]
	
	// CLASS ///////////////////////////////////////////////////////////////////////////////
	
	/**
	 * This example file provides you with an example of using the timeline object to 
	 * create a complex animation using the path object as a value map. Simply launch the 
	 * corresponding SWF file and click the mouse down to watch the animation play forwards, 
	 * then release the mouse button to watch it play in reverse.
	 * <p>
	 * Instructions for use:
	 * <p>
	 * Flash IDE users - this class is already set as the document class in the 
	 * corresponding FLA file for you. Simply open the FLA file and make any 
	 * changes to this file to experiment.
	 * <p>
	 * Non-Flash IDE users - Treat this class as your 'Main' class when you compile.
	 * 
	 * @see	com.boostworthy.animation.sequence.Timeline
	 * @see	com.boostworthy.geom.Path
	 */
	public class SequenceValueMapExample extends Sprite
	{
		// *********************************************************************************
		// CLASS DECLERATIONS
		// *********************************************************************************
		
		// CONSTANTS ///////////////////////////////////////////////////////////////////////
		
		/**
		 * Configures the color of the box sprite.
		 */
		protected const BOX_COLOR:uint       = 0x006666;
		
		/**
		 * Configures the width of the box sprite.
		 */
		protected const BOX_WIDTH:Number     = 60;
		
		/**
		 * Configures the height of the box sprite.
		 */
		protected const BOX_HEIGHT:Number    = 60;
		
		/**
		 * Configures the color of the circle sprite.
		 */
		protected const CIRCLE_COLOR:uint    = 0x006666;
		
		/**
		 * Configures the radius of the circle sprite.
		 */
		protected const CIRCLE_RADIUS:Number = 5;
		
		// MEMBERS /////////////////////////////////////////////////////////////////////////
		
		/**
		 * Holds an instance of the timeline object.
		 */
		protected var m_objTimeline:Timeline;
		
		/**
		 * Reference to the sprite which will contain the graph of the animation
		 * as a visual reference for this example.
		 */
		protected var m_spGraph:Sprite;
		
		/**
		 * Reference to the box sprite which will be created for the purpose
		 * of having something to animate.
		 */
		protected var m_spBox:Sprite;
		
		/**
		 * Reference to the circle sprite which will be created for the purpose
		 * of having a visual reference along the graph as the animation of the
		 * box sprite takes place.
		 */
		protected var m_spCircle:Sprite;
		
		/**
		 * Holds a path object to act as a value map for the box sprites animation. It
		 * will also be used to draw out the path as a visual reference for this example.
		 */
		protected var m_objPath:Path;
		
		// *********************************************************************************
		// EVENT HANDLERS
		// *********************************************************************************
		
		// CONSTRUCTOR /////////////////////////////////////////////////////////////////////
		
		/**
		 * Constructor.
		 */
		public function SequenceValueMapExample()
		{
			// Initialize this object.
			init();
		}
		
		// MOUSE EVENTS ////////////////////////////////////////////////////////////////////
		
		/**
		 * Called when the mouse is clicked down.
		 * 
		 * @param	objEvent	An object containing information about the mouse event.
		 */
		protected function onMouseDown(objEvent:MouseEvent):void
		{
			// Play the timeline forwards.
			m_objTimeline.play();
		}
		
		/**
		 * Called when the mouse is clicked up.
		 * 
		 * @param	objEvent	An object containing information about the mouse event.
		 */
		protected function onMouseUp(objEvent:MouseEvent):void
		{
			// Play the timeline backwards.
			m_objTimeline.playReverse();
		}
		
		// *********************************************************************************
		// INTERNAL
		// *********************************************************************************
		
		// INIT ////////////////////////////////////////////////////////////////////////////
		
		/**
		 * Initializes this object.
		 */
		protected function init():void
		{
			// Set all default property values.
			setDefaultValues();
			
			// Store a global reference to the stage.
			// This is needed for objects to have reference to the enter
			// frame event.
			Global.stage = stage;
			
			// Create a new timeline object.
			m_objTimeline = new Timeline(RenderMethod.TIMER, 60);
			
			// Create a new animation graph for the animations.
			createAnimationGraph();
			
			// Create a box sprite to animate.
			createBox();
			
			// Create a circle sprite to animate.
			createCircle();
			
			// Create animations for the box and circle sprites.
			createAnimation();
			
			// Add event listeners for the 'mouse down' and 'mouse up' events.
			stage.addEventListener(MouseEvent.MOUSE_DOWN, onMouseDown);
			stage.addEventListener(MouseEvent.MOUSE_UP, onMouseUp);
		}
		
		/**
		 * Applies all default values to the corresponding properties.
		 */
		protected function setDefaultValues():void
		{
			// Apply the default values.
			stage.scaleMode = StageScaleMode.NO_SCALE;
			stage.align     = StageAlign.TOP_LEFT;
		}
		
		// GRAPHICS ////////////////////////////////////////////////////////////////////////
		
		/**
		 * Creates a box sprite to animate.
		 */
		protected function createBox():void
		{
			// Create a new sprite and draw a box using the configured box settings.
			m_spBox      = new Sprite();
			m_spBox.name = "m_spBox";
			m_spBox.graphics.beginFill(BOX_COLOR);
			m_spBox.graphics.drawRect(-BOX_WIDTH / 2, -BOX_HEIGHT / 2, BOX_WIDTH, BOX_HEIGHT);
			
			// Add the box to the display container.
			addChild(m_spBox);
			
			// Start the box off in the center of the screen.
			m_spBox.x = Math.floor(stage.stageWidth / 2);
			m_spBox.y = Math.floor(stage.stageHeight / 2) + 50;
		}
		
		/**
		 * Creates a circle sprite to animate.
		 */
		protected function createCircle():void
		{
			// Create a new sprite and draw a circle using the configured circle settings.
			m_spCircle      = new Sprite();
			m_spCircle.name = "m_spCircle";
			m_spCircle.graphics.beginFill(CIRCLE_COLOR);
			m_spCircle.graphics.drawCircle(0, 0, CIRCLE_RADIUS);
			
			// Add the circle sprite to the graph sprite.
			m_spGraph.addChild(m_spCircle);
			
			// Position the circle sprite at the beginning of the path.
			m_spCircle.x = m_objPath.start.x;
			m_spCircle.y = m_objPath.start.y;
		}
		
		// ANIMATION ///////////////////////////////////////////////////////////////////////
		
		/**
		 * Create a new animation graph to act as a value map for the box sprite 
		 * and also server as a visual reference for the animation.
		 */
		protected function createAnimationGraph():void
		{
			// Create a new sprite to act as a container for the graph.
			m_spGraph = new Sprite();
			
			// Stage points to minimize calculations.
			var nY:Number = Math.floor(stage.stageHeight / 4);
			
			// Create a new path and use the API to add curves.
			m_objPath = new Path();
			m_objPath.moveTo(0, nY);
			m_objPath.curveTo(30,  nY - 50,  60,  nY);
			m_objPath.curveTo(90,  nY + 50,  120, nY);
			m_objPath.curveTo(180, nY - 100, 240, nY);
			m_objPath.curveTo(270, nY + 50,  300, nY);
			m_objPath.curveTo(330, nY - 50,  360, nY);
			
			// OPTIONAL
			// 
			// This step isn't required for animation, however I chose to draw
			// the path as well as a visual reference for this example.
			var objGraphics:Sprite = new Sprite();
			objGraphics.graphics.lineStyle(1, 0x555555);
			m_objPath.draw(objGraphics.graphics);
			m_spGraph.addChild(objGraphics);
			m_spGraph.x = Math.floor(stage.stageWidth / 2 - m_spGraph.width / 2);
			addChild(m_spGraph);
		}
		
		/**
		 * Creates the animation for the box sprite, but also creates an animation for the circle
		 * sprite to act as a visual for the status of the value map during the tween.
		 */
		protected function createAnimation():void
		{
			// OPTIONAL
			// 
			// This step adds a tween to animate the circle sprite along the path, thus
			// acting as a marker to visually display where the boxes values are coming from.
			m_objTimeline.addTween(new PathTween(m_spCircle, m_objPath, false, m_objPath.start.x, m_objPath.end.x, Transitions.SINE_IN_AND_OUT));
			
			// Tween the box sprites width and height using the path as a value map.
			m_objTimeline.addTween(new AdvancedTween(m_spBox, "width",  m_objPath, Transitions.SINE_IN_AND_OUT));
			m_objTimeline.addTween(new AdvancedTween(m_spBox, "height", m_objPath, Transitions.SINE_IN_AND_OUT));
		}
	}
}